#!/usr/bin/env python3

import sys
import os

try:
    import fastnlo as fnlo
except ModuleNotFoundError:
    print("fastNLO Python extension not found")
    sys.exit(77)

LIBDIR = os.environ["LIBDIR"]
DATADIR = f"{os.path.dirname(os.path.abspath(__file__))}/../data/check/"

TABLE_TYPE = ["fix", "flex"]
NODE_COUNTING = ["NodesPerBin", "NodeDensity"]
DISTANCE_MEASURE = ["slx", "lgx"]
SEED = [1000, 2000]

# fnlo.SetGlobalVerbosity(fnlo.DEBUG)

os.system("rm -rf output")
os.system(f"ln -sf {DATADIR}InclusiveNJets_InclusiveJets_fix.wrm InclusiveNJets_InclusiveJets_fix_slx.wrm")
os.system(f"ln -sf {DATADIR}InclusiveNJets_InclusiveJets_fix.wrm InclusiveNJets_InclusiveJets_fix_lgx.wrm")
os.system(f"ln -sf {DATADIR}InclusiveNJets_InclusiveJets_flex.wrm InclusiveNJets_InclusiveJets_flex_slx.wrm")
os.system(f"ln -sf {DATADIR}InclusiveNJets_InclusiveJets_flex.wrm InclusiveNJets_InclusiveJets_flex_lgx.wrm")


def generate_table(table_type: str, distance_measure: str, node_counting: str, seed: int):
    os.system(f"ln -sf {DATADIR}{table_type}_{distance_measure}_{node_counting}.str InclusiveNJets.str")
    print(f"Asserting that steering file InclusiveNJets.str -> {DATADIR}{table_type}_{distance_measure}_{node_counting}.str exists")
    assert os.path.exists("InclusiveNJets.str")
    os.system(
        "nlojet++ "
        "--calculate "
        "-c born "
        f"-s {seed} "
        f"--name {table_type}_{distance_measure}_{node_counting}_{seed:06d} "
        f" -u {LIBDIR}/fastnlo_interface_nlojet/libInclusiveNJets.la "
        "--max-event 10000 "
        "--save-after 10000 "
    )
    expected_outfile = "output/run-hhc-born-2jet.tab.gz"
    print(f"Asserting that there is a table under {expected_outfile}")
    assert os.path.exists(expected_outfile)
    os.system(f"mv {expected_outfile} output/{table_type}_{distance_measure}_{node_counting}_{seed:06d}.tab.gz")


for table_type in TABLE_TYPE:
    for distance_measure in DISTANCE_MEASURE:
        for node_counting in NODE_COUNTING:
            for seed in SEED:
                generate_table(table_type, distance_measure, node_counting, seed)
            os.system(
                f"fnlo-tk-merge2 output/{table_type}_{distance_measure}_{node_counting}_??????.tab.gz "
                f"output/{table_type}_{distance_measure}_{node_counting}_merged.tab.gz")

        for seed in SEED:
            filename_0 = f"output/{table_type}_{distance_measure}_{NODE_COUNTING[0]}_{seed:06d}.tab.gz"
            filename_1 = f"output/{table_type}_{distance_measure}_{NODE_COUNTING[1]}_{seed:06d}.tab.gz"
            table_0 = fnlo.fastNLOTable(filename_0)
            table_1 = fnlo.fastNLOTable(filename_1)
            print(f"Asserting equal: {filename_0} <-> {filename_1}")
            assert table_0.IsEquivalent(table_1, 1e-8)

        filename_0 = f"output/{table_type}_{distance_measure}_{NODE_COUNTING[0]}_merged.tab.gz"
        filename_1 = f"output/{table_type}_{distance_measure}_{NODE_COUNTING[1]}_merged.tab.gz"
        table_0_merged = fnlo.fastNLOTable(filename_0)
        table_1_merged = fnlo.fastNLOTable(filename_1)
        print(f"Asserting equal: {filename_0} <-> {filename_1}")
        assert table_0_merged.IsEquivalent(table_1_merged, 1e-8)
